<template>
  <div class="info-dispatch">
    <van-search
      class="page-search-container"
      v-model="keyword"
      placeholder="请输入搜索关键词"
      :show-action="sysUserInfo.role === 'Company'"
      @search="onSearch"
    >
      <template #action>
        <div @click="showFilter = true">筛选</div>
      </template>
    </van-search>
    <van-action-sheet v-model="showFilter" title="筛选" @closed="handleClose">
      <div class="filter-content">
        <p>状态</p>
        <van-checkbox-group v-model="onDuty">
          <block-checkbox :name="true">在岗</block-checkbox>
          <block-checkbox :name="false">离岗</block-checkbox>
        </van-checkbox-group>
        <p>人员</p>
        <van-checkbox-group v-model="result">
          <block-checkbox name="a">全部</block-checkbox>
          <block-checkbox name="b">负责人</block-checkbox>
        </van-checkbox-group>
        <p v-if="stationType === 'TemporarySecurity'">类型</p>
        <van-checkbox-group v-model="stationTypes" v-if="stationType === 'TemporarySecurity'">
          <block-checkbox v-for="item in allStationTypes" :key="item.code" :name="item.code"> {{ item.name }}</block-checkbox>
        </van-checkbox-group>
      </div>
    </van-action-sheet>
    <van-list ref="results" class="content">
      <van-cell v-if="showAllSelect" class="all">
        <span class="count">当前共选择 {{ selectedCount }} 个</span>
        <van-checkbox v-model="checked" @change="handleCheckAll">全选</van-checkbox>
      </van-cell>
      <div class="employee-box" v-for="(item, index) in employeeList" :key="index" @click="select(item)">
        <van-checkbox class="checkbox" v-model="item.checked" :disabled="item.dutyStatus !== 'SignIn'"></van-checkbox>
        <avatar :size="40" @click.stop.native="toEmployee(item)">
          <div class="per-name">
            {{ item.nameSlice }}
          </div>
        </avatar>
        <div class="info">
          <div class="title">{{ item.name }}（{{ item.dutyStatusText }}）</div>
          <div class="detail">
            <span class="label">所属公司：</span>
            <span class="value">{{ item.companyName }}</span>
          </div>
          <div class="detail">
            <span class="label">所属{{ item.isTemporarySecurity ? "岗点" : item.isSecurity ? "驻勤点" : "社区警务" }}：</span>
            <span class="value">{{ item.securityStationName }}</span>
          </div>
          <div class="detail">
            <span class="label">联系方式：</span>
            <span v-if="!item.concatVisible" class="value">{{ item.contactHide }}</span>
            <span v-if="item.concatVisible" class="value">{{ item.contact }}</span>
            <span class="visible" @click.stop="item.concatVisible = !item.concatVisible">{{
              item.concatVisible ? "隐藏" : "显示"
            }}</span>
          </div>
          <div class="detail">
            <span class="label">证件号码：</span>
            <span v-if="!item.idVisible" class="value">{{ item.idCardNoHide }}</span>
            <span v-if="item.idVisible" class="value">{{ item.idCardNo }}</span>
            <span class="visible" @click.stop="item.idVisible = !item.idVisible">{{ item.idVisible ? "隐藏" : "显示" }}</span>
          </div>
          <div class="detail">
            <span class="label">人员类型：</span>
            <span class="value">{{ getStationTypesName(item.stationType) }}</span>
          </div>
        </div>
      </div>
    </van-list>
    <div class="load-more">
      <van-button type="info" style="width: 90px; margin: 0 auto" v-if="!finished" block @click="onLoad" :disabled="loading">
        加载更多
      </van-button>
    </div>
    <div class="dispatch" v-if="employeeList.length > 0">
      <div class="title">当前共选择{{ selectedCount }}人</div>
      <div class="btn-list">
        <van-button class="btn" plain hairline size="mini" type="info" @click="openGroupFormDialog">文字调度</van-button>
        <van-button class="btn" plain hairline size="mini" type="info" @click="dispatch('audio')">语音调度</van-button>
        <van-button class="btn" plain hairline size="mini" type="info" @click="dispatch('video')">视频调度</van-button>
        <van-button class="btn" plain hairline size="mini" type="info" @click="dispatch('videoLocalMute')">视频上拉</van-button>
      </div>
    </div>
    <van-empty v-else description="暂无数据" />
    <!-- 即时通信 -->
    <current-conversation :visible.sync="showConversation"></current-conversation>
    <!-- 音视频 -->
    <call-layer
      ref="callLayer"
      :member-list.sync="memberList"
      :type="callType"
      :group-call="groupCall"
      :yardman-type="yardmanType"
      :audio="stream.audio"
      :video="stream.video"
      @showMembers="showMembers"
      @close="onCloseCalling"
    ></call-layer>
    <!-- 音视频成员 -->
    <calling-members
      :visible.sync="callingMembersVisible"
      :member-list.sync="memberList"
      :all-members.sync="employeeList"
      :type="callType"
      :yardman-type="yardmanType"
      @sendVideoRoomMsg="sendVideoRoomMsg"
    ></calling-members>
  </div>
</template>

<script>
import { Search, List, Empty, Button, CheckboxGroup, Checkbox, Toast, Notify, ActionSheet, Cell } from "vant";
import Avatar from "@components/avatar.vue";
import { textSlice } from "@/api/common";
import { getEmployees } from "@/api/employee-controller";
import CurrentConversation from "@/components/timApp/conversation/current-conversation.vue";
import CallLayer from "@/components/timApp/trtc/calling-index.vue";
import CallingMembers from "@/components/timApp/trtc/calling-members.vue";
import tim from "@/tim";
import _webSocket from "@/utils/websocket";
import { mapState, mapActions } from "vuex";
import BlockCheckbox from "@/components/blockCheckbox/index.vue";

export default {
  name: "StationEmployees",
  components: {
    CallingMembers,
    CallLayer,
    CurrentConversation,
    [Search.name]: Search,
    [List.name]: List,
    [Empty.name]: Empty,
    [Checkbox.name]: Checkbox,
    [CheckboxGroup.name]: CheckboxGroup,
    [Button.name]: Button,
    [Toast.name]: Toast,
    [Notify.name]: Notify,
    [ActionSheet.name]: ActionSheet,
    [Cell.name]: Cell,
    Avatar,
    BlockCheckbox,
  },
  data() {
    return {
      keyword: "",
      employeeList: [],
      checked: false,
      memberList: [],
      loading: false,
      finished: false,
      pageIndex: 1,
      showConversation: false,
      callType: "video",
      groupCall: false,
      //调度类型：1视频调度 2 视频上拉：除了视频上拉传2，其他音频调度与视频调度都传1
      yardmanType: "1",
      stream: {
        audio: true,
        video: true,
      },
      callingMembersVisible: false,
      showFilter: false,
      result: [],
      onDuty: [],
      stationTypes: [],
    };
  },
  computed: {
    ...mapState({
      roomId: (state) => state.trtc.roomId,
      allStationTypes: (state) => state.app.stationTypes,
      sysUserInfo: (state) => state.user.sysUserInfo,
    }),
    stationId() {
      return this.$route.query.stationId;
    },
    selectedCount() {
      return this.employeeList.filter((x) => x.checked).length;
    },
    showAllSelect() {
      return this.employeeList.filter((item) => item.dutyStatus === "SignIn").length > 0;
    },
    stationType() {
      return this.$route.query.stationType;
    },
  },
  methods: {
    ...mapActions(["setAllStationTypes"]),
    //ws初始化
    initWs() {
      const dispatchToken = localStorage.getItem("dispatchToken");
      if (!dispatchToken) return false;
      this.webSocketUrl = `/websocket/police_incidents?dispatchToken=${dispatchToken}`;
      this.ws = _webSocket({
        url: this.webSocketUrl,
        onMessage: this.onMessage,
        heart: {
          timer: 5000,
          message: "keep alive",
        },
        isReconnect: true,
        //onClose: this.onClose,
      });
    },
    onClose() {
      this.initWs();
    },
    onMessage(evt) {
      // console.log("ws data ===", evt);
      if (evt) {
        if (evt.data.includes("keep alive")) {
          //心跳
        } else {
          const data = JSON.parse(evt.data);
          console.log("ws data ===", data);
          const { type, message } = data;
          if (type === "PoliceIncidents") {
            const content = message.contents[0];
            if (content.category === "Sos") {
              // TODO 处理SOS
            } else {
              // TODO 处理警情上报
            }
          } else if (type === "TRTC") {
            const content = message.contents[0];
            console.log(content);
            const { action, tencentUserId, roomId } = content;
            // 如果不是本人的房间直接忽略
            if (this.roomId && roomId.toString() === this.roomId.toString()) {
              if (action === "handUp" || action === "cheese") {
                // this.$message.info("有1人举手");
                Toast({
                  message: "有1人举手",
                  position: "top",
                  icon: require("@/assets/images/conversation/移动端举手.png"),
                });
              }
              this.memberList.forEach((member) => {
                if (member.tencentUserId === tencentUserId) {
                  member.action = action;
                }
              });
              console.log(this.memberList);
            }
          }
        }
      }
    },
    onSearch() {
      this.initLoading();
      this.onLoad();
    },
    initLoading() {
      this.pageIndex = 1;
      this.employeeList = [];
      this.finished = false;
    },
    getStationIds() {
      if (this.stationId) {
        return this.stationId.split(",");
      }
    },
    onLoad() {
      this.loading = true;
      let responsibleOfStationIds;
      if (this.sysUserInfo.orgInstitutional === "0201" && this.result.length > 0) {
        responsibleOfStationIds = this.result.includes("a") ? null : [...this.getStationIds()];
      } else {
        responsibleOfStationIds = null;
      }
      let onDutyValue = null;
      if (this.onDuty[0] === true && this.onDuty[1] === false) {
        onDutyValue = null;
      } else if (this.onDuty[0] === true) {
        onDutyValue = true;
      } else if (this.onDuty[0] === false) {
        onDutyValue = false;
      }
      getEmployees({
        pageIndex: this.pageIndex,
        pageSize: 10,
        searchModel: "SecurityStation",
        securityStationIds: [...this.getStationIds()],
        keyword: this.keyword,
        deskTypes: this.$route.query.stationType === "TemporarySecurity" ? ["TemporarySecurity"] : ["Normal", "Proprietor"],
        onDuty: onDutyValue,
        stationTypes: this.stationTypes,
        responsibleOfStationIds: responsibleOfStationIds,
      })
        .then((res) => {
          this.employeeList = [...this.employeeList, ...this.resetEmployeeList(res)].map((x) => {
            return {
              dutyStatus: "",
              dutyStatusText: "",
              isSecurity: x.rts.includes("Security"),
              isTemporarySecurity: x.rts.includes("Security") && x.rts.includes("TemporarySecurity"),
              ...x,
            };
          });
          // 加载状态结束
          this.loading = false;
          // 数据全部加载完成
          if ([...res].length <= 0) {
            this.finished = true;
          } else {
            this.pageIndex++;
          }
        })
        .catch((err) => {
          console.log(err);
        })
        .finally(() => {
          if (this.employeeList.length > 5) {
            this.$nextTick(() => {
              this.scrollToBottom();
            });
          }
        });
    },
    scrollToBottom() {
      const container = this.$refs.results.$el;
      console.log(container);
      container.scrollTop = container.scrollHeight;
    },
    // 重置字段
    resetEmployeeList(list) {
      return list.map((item) => {
        item.active = false;
        item["nameSlice"] = item.name.slice(-2);
        item["idCardNoHide"] = this.parseStringToStar(item.idCardNo);
        item["idVisible"] = false;
        item["contactHide"] = this.parseStringToStar(item.contact);
        item["concatVisible"] = false;
        item["companyNameSlice"] = textSlice(item.companyName);
        item["securityStationNameSlice"] = textSlice(item.securityStationName);
        item["type"] = "employee";
        item["isPhone"] = false;
        item["isIdCardNo"] = false;
        item["checked"] = false;
        return item;
      });
    },
    parseStringToStar() {
      return "**************";
      // return str.length > 2 ? str.substr(0, 2) + new Array(str.length - 2).join("*") + str.substr(-2) : str;
    },
    select(item) {
      const { dutyStatus, checked } = item;
      if (dutyStatus === "SignIn") {
        item.checked = !checked;
      }
    },
    handleCheckAll(value) {
      console.log(value);
      //  this.resResult.forEach((item) => (item.checked = value));
      this.employeeList.forEach((employee) => {
        const { dutyStatus } = employee;
        if (dutyStatus === "SignIn") {
          employee.checked = value;
        }
      });
    },
    toEmployee(item) {
      this.$router.push({
        path: "/employeeInfo",
        query: {
          queryId: item.id,
        },
      });
    },
    dispatch(type) {
      console.log(type);
      const memberList = this.employeeList.filter((x) => x.checked);
      this.onDispatch({ type, memberList });
    },
    initCalling(groupCall) {
      return new Promise((resolve) => {
        this.groupCall = groupCall;
        resolve();
      });
    },
    /**
     * @调度前参数设置
     * @param type
     * @returns {Promise<void>}
     */
    setCallProps(type) {
      if (type === "video") {
        this.stream.video = true;
        this.stream.audio = true;
        this.callType = "video";
        this.yardmanType = "1";
      } else if (type === "audio") {
        this.stream.video = false;
        this.stream.audio = true;
        this.callType = "audio";
        this.yardmanType = "1";
      } else if (type === "videoLocalMute") {
        this.stream.video = false;
        this.stream.audio = true;
        this.callType = "video";
        this.yardmanType = "2";
      }
      return Promise.resolve();
    },
    /**
     * @多人调度
     * @param type
     * @param memberList
     */
    onDispatch({ type, memberList }) {
      this.memberList = [...memberList];
      this.memberList.forEach((member) => {
        member.state = "CONNECTING";
        member.action = "";
      });
      if (this.memberList.length > 0) {
        this.setCallProps(type).then(() => {
          this.startVideoDispatch(this.memberList.length > 1);
        });
      } else {
        Toast.fail("至少选择一名被调度人");
      }
    },
    startVideoDispatch(groupCall) {
      // this.$bus.$emit("video-call");
      this.initCalling(groupCall).then(() => {
        // this.$refs.callLayer.showCalling = true;
        this.$bus.$emit("video-call");
      });
    },
    videoLocalMuteCall() {
      this.memberList = [this.employee];
      this.allMembers = [];
      this.memberList.forEach((member) => {
        member.state = "CONNECTING";
        member.action = "";
        // member.id = member.employeeId
      });
      this.stream.video = false;
      this.stream.audio = true;
      this.yardmanType = "2";
      this.callType = "video";
      return Promise.resolve();
    },
    videoLocalMute() {
      console.log("上拉", this.info);
      this.videoLocalMuteCall().then(() => {
        this.$bus.$emit("video-call");
      });
    },
    /**
     * @视频调度1v1
     */
    dispatchOneVideo() {
      this.memberList = [this.employee];
      this.memberList.forEach((member) => {
        member.state = "CONNECTING";
        member.action = "";
      });
      this.setCallProps("video").then(() => {
        this.startVideoDispatch(false);
      });
    },
    /**
     * @语音调度1v1
     */
    dispatchOneAudio() {
      this.memberList = [this.employee];
      this.memberList.forEach((member) => {
        member.state = "CONNECTING";
        member.action = "";
      });
      this.setCallProps("audio").then(() => {
        this.startVideoDispatch(false);
      });
    },
    openGroupFormDialog() {
      console.log(this.employeeList);
      const members = this.employeeList.filter((x) => x.checked);
      if (members.length > 0) {
        if (members.length > 1) {
          // 群聊
          this.createGroupConversation(members);
        } else {
          // 单人聊天
          this.character(members[0]);
        }
      } else {
        Toast.fail("至少选择一名被调度人");
      }
    },
    /**
     * @文字调度事件-单人
     */
    character(info) {
      const { tenantImUserId } = info;
      tim.getConversationProfile(`C2C${tenantImUserId}`).then(({ data }) => {
        console.log("getConversationProfile---", data);
        this.$store.commit("updateCurrentConversation", {
          ...data.conversation,
          ...{
            info,
          },
        });
        const { conversationID } = data.conversation;
        this.$store.dispatch("getMessageList", conversationID);
        this.showConversation = true;
      });
    },
    /**
     * @设置群聊创建条件
     * @param members
     * @returns {{memberList: *}}
     */
    getOptions(members) {
      return {
        type: this.TIM.TYPES.GRP_WORK,
        name: "指挥调度",
        memberList: members.map(({ tenantImUserId }) => {
          return { userID: tenantImUserId + "" };
        }),
      };
    },
    /**
     * @创建IM文字群聊
     * @param members
     */
    createGroupConversation(members) {
      console.log(members);
      const groupOptions = this.getOptions(members);
      tim
        .createGroup({ ...groupOptions })
        .then((imResponse) => {
          Toast.success(`群组：【${imResponse.data.group.name}】创建成功`);
          this.showGroupConversation(imResponse.data);
        })
        .catch((error) => {
          Toast.fail(error.message);
        });
    },
    /**
     * @打开群聊弹框
     * @param group
     */
    showGroupConversation({ group }) {
      console.log("group data ===", group);
      tim.getConversationProfile(`GROUP${group.groupID}`).then(({ data }) => {
        console.log(data);
        this.$store.commit("updateCurrentConversation", data.conversation);
        const { conversationID } = data.conversation;
        this.$store.dispatch("getMessageList", conversationID);
        console.log(this.currentConversation);
        this.showConversation = true;
      });
    },
    showMembers() {
      // 可选择的人就是调度弹框内的人，checked的人是已经在调度的人，active的人就是要加入调度的人
      console.log(this.employeeList);
      this.callingMembersVisible = true;
    },
    sendVideoRoomMsg(roomId, memberList, UseList) {
      this.callingMembersVisible = false;
      this.$refs.callLayer.sendRoomMessage(roomId, memberList, UseList);
    },
    onCloseCalling() {
      this.callingMembersVisible = false;
    },
    handleClose() {
      this.onSearch();
    },
    getStationTypesName(stationType) {
      if (!stationType) return "";
      if (this.allStationTypes.length <= 0) return "";
      let stationTypesName = "";
      const type = this.allStationTypes.filter((item) => item.code === stationType);
      if (type && type.length > 0) {
        stationTypesName = type[0].name;
      }
      return stationTypesName;
    },
  },
  created() {
    this.setAllStationTypes();
    this.onLoad();
    this.initWs();
  },
};
</script>

<style scoped lang="less">
.info-dispatch {
  // .van-checkbox__icon {
  //   height: 100%;
  // }
  .load-more {
    padding: 10px;
  }
  .all {
    .van-cell__value {
      display: flex;
      align-items: center;
      justify-content: space-between;
    }
  }
  .content {
    padding: 24px 20px;
    overflow-y: auto;
    overflow-x: hidden;
    max-height: calc(100vh - 86px - 46px - 44px - 60px);
    box-sizing: border-box;

    .employee-box {
      width: 100%;
      height: 165.5px;
      border-bottom: 1px solid #d8d8d8;
      display: flex;
      flex-direction: row;
      padding-top: 12px;
      justify-content: space-between;
      position: relative;
      .checkbox {
        position: absolute;
        right: 0;
        top: 50%;
        transform: translate(-50%, 0);
      }
      .per-name {
        height: 40px;
        width: 40px;
        background: #0e71ff;
        color: #fff;
        font-size: 12px;
        display: flex;
        align-items: center;
        justify-content: center;
      }
      .info {
        width: 287px;
        padding-left: 12px;
        .title {
          font-size: 14px;
          font-family: PingFang SC-Medium, PingFang SC, serif;
          font-weight: 500;
          color: #303133;
          line-height: 22px;
          margin-bottom: 2px;
        }
        .detail {
          width: 100%;
          display: flex;
          align-items: center;
          margin-bottom: 10px;
          &:last-child {
            margin-bottom: 0;
          }
          .label {
            font-size: 12px;
            font-family: PingFang SC-Regular, PingFang SC, serif;
            font-weight: 400;
            color: #909399;
          }
          .value {
            font-size: 12px;
            font-family: PingFang SC-Regular, PingFang SC, serif;
            font-weight: 400;
            color: #303133;
            &-link {
              font-size: 12px;
              font-family: PingFang SC-Regular, PingFang SC, serif;
              font-weight: 400;
              color: #0f71ff;
              text-decoration: underline;
            }
          }
          .visible {
            font-size: 12px;
            font-family: PingFang SC-Regular, PingFang SC, serif;
            font-weight: 400;
            color: #0f71ff;
            margin-left: 2px;
          }
        }
      }
    }
  }
  .filter-content {
    padding: 24px 20px;
    .van-checkbox-group {
      display: flex;
      flex-wrap: wrap;
    }
  }
  .dispatch {
    width: 100%;
    padding: 8px 20px;
    box-sizing: border-box;
    position: absolute;
    bottom: 0;
    left: 0;
    .title {
      font-size: 12px;
      line-height: 22px;
      font-family: PingFang SC-Medium, PingFang SC, serif;
      font-weight: 500;
      color: #3d3d3d;
      margin-bottom: 10px;
    }
    .btn-list {
      display: flex;
      flex-direction: row;
      flex-wrap: nowrap;
      .btn {
        margin-bottom: 14px;
        margin-right: 10px;
        margin-left: 0;
        width: 78px;
        //border-radius: 6px 6px 6px 6px;
      }
    }
  }
}
</style>
